unit DHYTModule;

interface
uses Transformation;

function DHYTInit(X,Y:longword):TTransError; cdecl;
function DHYTFinalize : TTransError; cdecl;
procedure DHYTGetEM(Mask:pointer; Energy:pointer) cdecl;
function DHYTGetBVal(FnX,FnY,X,Y:longword):TFloat; cdecl;
function DHYTTransform(Dir:TTransDirection; Data:pointer):TTransError; cdecl;
function DHYTGetP(ID:longword;var Value:pointer; var Len:longword):TTransError; cdecl;
function DHYTSetP(ID:longword;Value:pointer;Len:longword):TTransError; cdecl;
function DHYTGetConf(var Value:pointer):longword; cdecl;
procedure DHYTSetConf(Value:pointer;Len:longword); cdecl;
function DHYTGetQuantM(X,Y,Width,Height:longword):TFloat; cdecl;
function DHYTGetStatus : TTransStatus; cdecl;


var
  DHYT : TTransformation = (
    Name : 'DHYT';
    Description : 'Discrete Hartley Transformation';
    ID : 3;
    Version : $100;
    Public : true;
    Init : DHYTInit;
    Finalize : DHYTFinalize;
    GetEnergyMatrix : nil;//DHYTGetEM;
    GetBasisValue : DHYTGetBVal;
    Transform : DHYTTransform;
    GetProperty : DHYTGetP;
    SetProperty : DHYTSetP;
    GetConfig : DHYTGetConf;
    SetConfig : DHYTSetConf;
    Configure : nil;
    GetQuantMatrixItem : DHYTGetQuantM;
    Status : DHYTGetStatus;
  );


implementation
uses DOT;

var
  Basis1,Basis2 : TShapeData;
  sqrtN1N2 : TFloat;


function DHYTInit;
var i:integer;
begin
  //Initialize internal transformation to DHYT
  DHYT.LastError:=TTransError(DOT.Initialize(X,Y,dmDHYT));
  if DHYT.LastError=teOK then begin
    DHYT.X:=X;
    DHYT.Y:=Y;
    sqrtN1N2:=sqrt(X*Y);
    try
      SetLength(Basis1,X);
      SetLength(Basis2,Y);
      //Make Basis generator arrays
      for i:=0 to X-1 do Basis1[i]:=cos(2*i*Pi/X)+sin(2*i*Pi/X);
      for i:=0 to Y-1 do Basis2[i]:=cos(2*i*Pi/Y)+sin(2*i*Pi/Y);
    except
      DOT.Finalize;
      DHYT.LastError:=teNotEnoughMem;
    end;
  end;
  Result:=DHYT.LastError;
end;


function DHYTFinalize;
begin
  if DOT.GetMode<>dmDHYT then begin DHYT.LastError:=teIncorrectMode; Result:=teIncorrectMode; exit end;
  DHYT.LastError:=TTransError(DOT.Finalize);
  SetLength(Basis1,0);
  SetLength(Basis2,0);
  Result:=DHYT.LastError;
end;


procedure DHYTGetEM;
var
    iTemp, iEnergy : TShapeData;
    norma, AC : TFloat;
    i,j : integer;
    halfX, halfY : word;
begin
  if DOT.GetMode<>dmDHYT then begin DHYT.LastError:=teIncorrectMode; exit; end;
  iTemp:=Mask; iEnergy:=Energy;
  norma:=sqrt(DHYT.X*DHYT.Y);
  DHYT.LastError:=DHYT.Transform(tdForward,Mask);
  DHYT.LastError:=DHYT.Transform(tdForward,Mask);
  AC:=iTemp[0]/(4*norma);
  halfX:=DHYT.X shr 1; halfY:=DHYT.Y shr 1;
  for j:=0 to halfY-1 do
    for i:= 0 to halfX-1 do begin
      iEnergy[j*DHYT.X+i] := AC + iTemp[(2*j)*DHYT.X+(2*i)]/norma;
      iEnergy[j*DHYT.X+i+halfX] := iEnergy[j*DHYT.X+i];
      iEnergy[(j+halfY)*DHYT.X+i] := iEnergy[j*DHYT.X+i];
      iEnergy[(j+halfY)*DHYT.X+i+halfX] := iEnergy[j*DHYT.X+i];
    end;
end;


function DHYTGetBVal;
begin
  if DOT.GetMode<>dmDHYT then begin DHYT.LastError:=teIncorrectMode; Result:=0; exit end;
  if DOT.State=dsClosed then begin DHYT.LastError:=teClosed; Result:=0; exit end;
  Result:=(Basis1[(X*FnX) mod DHYT.X]
          *Basis2[(Y*FnY) mod DHYT.Y])/sqrtN1N2;
end;


function DHYTTransform;
begin
  if DOT.GetMode<>dmDHYT then begin DHYT.LastError:=teClosed; Result:=teIncorrectMode; exit end;
  if DOT.State=dsClosed then begin DHYT.LastError:=teClosed; Result:=teClosed; exit end;
  DHYT.LastError:=TTransError(DOT.Transform(TDOTDirection(Dir),nil,Data));
  Result:=DHYT.LastError;
end;


function DHYTGetP;
begin
  Value:=nil;
  Len:=0;
  Result:=tePropertyNotExists;
end;


function DHYTSetP;
begin
  Result:=tePropertyNotExists;
end;

function DHYTGetConf;
begin
  Value:=nil;
  Result:=0;
end;


procedure DHYTSetConf;
begin

end;


function DHYTGetQuantM;
begin
  Result:=1;
end;


function DHYTGetStatus : TTransStatus;
begin
  Result:=TTransStatus(DOT.State);
end;


end.
