unit Transformation;

interface
uses Windows;

const
  HDR_VER = $100;

type
  TFloat = real;
  TShapeData = array of TFloat;

  TTransError = (teOK, teNotEnoughMem, teBusy, teIncorrectMode, teOpened, teClosed,
                 teIncorrectSize, teInputDataSizeError, tePropertyNotExists);
  TTransDirection = (tdForward, tdInverse);
  TTransStatus = (tsClosed, tsReady, tsComputing, tsFail);


  TTransInit = function(X,Y:longword):TTransError; cdecl;
  TTransFinalize = function : TTransError; cdecl;
  TTransTransform = function(Dir:TTransDirection; Data:pointer):TTransError; cdecl;
  TTransGetProperty = function(ID:longword;var Value:pointer; var Len:longword):TTransError; cdecl;
  TTransSetProperty = function(ID:longword;Value:pointer;Len:longword):TTransError; cdecl;
  TTransConfigure = procedure; cdecl;
  TTransGetEnergyMatrix = procedure (Mask:pointer; Energy:pointer) cdecl;
  TTransGetBasisValue = function (FnX,FnY,X,Y:longword):TFloat; cdecl;
  TTransGetConfig = function(var Value:pointer):longword; cdecl;
  TTransSetConfig = procedure(Value:pointer;Len:longword); cdecl;
  TTransGetQuantMatrixItem = function (X,Y,Width,Height:longword):TFloat; cdecl;
  TTransGetStatus = function : TTransStatus; cdecl;

  TTransformation = packed record
    Name : PChar;  // Transfromation Name
    Description : PChar; //Transformation description
    ID : longword;  //unique ID of transformation
    Version : word;  //version (higher byte is major, lowwer is minor version)
    public : boolean;  //is transformation a public transformation ?
    Init : TTransInit;  //initialize transformation to pixel array X x Y
    Finalize : TTransFinalize;  //Finalize transformation
    GetEnergyMatrix : TTransGetEnergyMatrix;  //calculates energy matrix to array from parameters. If is NIL, application calculates this matrix with classis slow algorithm
    GetBasisValue : TTransGetBasisValue; //retrieves normalized value of basis function FnX,FnY at X,Y
    Transform : TTransTransform;  //Transform input data
    GetProperty : TTransGetProperty;  //Get transformation specific property
    SetProperty : TTransSetProperty;  //Set transformation specivic property
    GetConfig : TTransGetConfig;  //Get current configuration of codec (this config goes to output file)
    SetConfig : TTransSetConfig;  //Set configuration of coded (this is from input file)
    Configure : TTransConfigure;  // opens configuration dialog
    GetQuantMatrixItem : TTransGetQuantMatrixItem; //get quantization matrix element for basis function X,Y
    Status : TTransGetStatus; //transformation machine status
    LastError : TTransError;  //last error
    X,Y : longword;  //data size
  end;
  PTransformation = ^TTransformation;

  TModuleInit = procedure; cdecl;
  tModuleFinalize = procedure; cdecl;
  TGetTransformation = function (Which : longword) : PTransformation; cdecl;
  TFindTransformation = function(modID:longword; transID:longword):PTransformation; cdecl;

  TModuleHeader = packed record
    hdrVersion : word;  //Header version. Must filled by constant HDR_VER
    Name : PChar;  //name of module
    Description : PChar;  //description of module
    Author : PChar;  //Author of module
    ID : longword;  //Module ID
    Version : word;  //Module version
    HANDLE : HINST; //Handle of DLL Module
    Init : TModuleInit; //initialize module;
    Finalize : TModuleFinalize; //finalize module
    GetTransformation : TGetTransformation; //Module entry point, enumerates transformations
    FindTransformation : TFindTransformation; // is filled by system after loading the module.
                                              // Module can use this for communicate with other transformations
    AppVersion : word; //is filled by systetm with his version
  end;
  PModuleHeader = ^TModuleHeader;

  TGetModuleHeader = function : PModuleHeader;

implementation

end.
